#-----------------------------------------------------
# alloplot1 - to plot the urban allometric datasets
# Rob MacKenzie
# May 2018
# Oct 2019 - annotations on residuals plot
#-----------------------------------------------------

# reset all
rm(list=ls())
frame() # completes current plot
options("scipen"=10, "digits" = 4)  #defines how to present large and small numbers

#load the lattice graphics package for extra functionality (e.g. xyplot)
library(lattice, biglm)
setwd("U:/data/research/R_work")

#Useful number-from-string extraction
#library(stringr)
numextract <- function(string){ 
  str_extract(string, "\\-*\\d+\\.*\\d*")
} 

# input work data
#city_test<-read.csv("city_test.csv") #short subsection of data for testing
city<-read.csv("UrbanData_revised.csv",na.strings = "#N/A") #na.strings strips out the 'no data' chars

#subset the data into working data frame ...
city_sub<-subset(city, Total.residents>5e4) #...using absolute population thresholds


# check columns input correctly
names(city_sub)
head(city_sub)
str(city_sub)

#current parameter being analysed (eg, 3 = area; 8 = rd length; 10 = NOx)
param.col = 10 
city_test<-subset(city_sub, city_sub[,param.col]>0) #...excluding zeros and no data -999s
#str reports the structure of the dataset - useful for spotting numerical arrays that contain chars etc
#variables for plotting must be "int" or "num"
#NB: subsetting does not change the variable type - eg if read-in as a factor will stay a factor
str(city_test)

#plotting: use plot.formula, tilde to define the stat model 
plot(city_test[,param.col] ~ Total.residents, data = city_test,
     #set log axes 
     log = "xy",
     xlab = list("Population", cex = 1.),
     #The y-axis label is the xth (where x is param.col) element of the names in city_test
#     ylab = list(names(city_test)[param.col], cex = 1.5))
     #prettify the axes labels if need be
     ylab = list(expression('Log'['10']*'NO'['x']~'emissions'), cex = 1.)
      )
#log transforms
log.param=log10(city_test[,param.col]) #transform parameter
log.pop=log10(city_test$Total.residents) #transform x-axis independent variable
#linear model
lin.sum<-summary(lm(log.param ~ log.pop)) #summary of the linear model
alfa<-round(lin.sum$coefficients[2,1:2], digits = 3) #slope and error on slope, rounded to 3 sig figs

#add line of best fit and slope
abline(coef = coef(lin.sum))
#legend("topleft", legend=round(alpha,digits = 3))
#alternative way of writing out the slope, as text on the graph at position given by first two elements
text(mean(city_test$Total.residents),ceiling(max(city_test[,param.col])),
     paste("slope =",alfa[1],"+/-",alfa[2]))
#write out the r-square and sample size (n)
text(mean(city_test$Total.residents),ceiling(0.5*max(city_test[,param.col])),
     paste("r-squared =",round(lin.sum$adj.r.squared, digits = 2),"; n = ",dim(city_test)[1]))



#The area vs population graph needs special treatment
urb.form <-{-0.5*log10(city_test$Area.km2)} #transform area parameter and convert to sqm
area.grf<-data.frame(city_test$Total.residents, urb.form, log.param)

#Plot the scatter and line
plot(urb.form ~ city_test.Total.residents, data = area.grf,
     log = "x",
     xaxt = "n", # suppresses default x-axis
     #The y-axis label is the xth (where x is param.col) element of the names in city_test
     #ylab = list(paste("-0.5Log10(",names(city_test)[param.col],")"), cex = 1.5)
     ylab = list(expression('-0.5Log'['10']*'Area'), cex = 1.),
     xlab = list("Population", cex = 1.)
     #xlab = ""
     )
#Find the linear model; put a summary in urb.sum
urb.sum.area<-summary(lm(urb.form ~ log.pop)) #summary of the linear model
alfa.area<-round(urb.sum.area$coefficients[2,1:2], digits = 3) #slope and error on slope, rounded to 3 sig figs
#add line of best fit and slope
abline(coef = coef(urb.sum.area))
#Write out the slope and error on slope; first two elements give position on graph
text(mean(city_test$Total.residents),(min(urb.form)+0.2),
     paste("Slope =",alfa.area[1],"+/-",alfa.area[2]))
#write out the r-square and sample size, n
text(mean(city_test$Total.residents),min(urb.form),
     paste('r-squared = ',round(urb.sum.area$adj.r.squared, digits = 2),"; n = ",dim(city_test)[1])
     )
#re-draw the x-axis to be at top - but how to give it a title?
axis(side = 3)

#Plot the projected concentration scatter and line
uwind<-2
pbl<-1.e2
hu<-pbl*uwind
conc.form<-log.param+urb.form+log10(100/(3.1536*hu)) #eqn s3, with const to take care of units and u.h~5e3
conc.grf<-data.frame(city_test$Total.residents, conc.form)

frame() # completes current plot
plot(conc.form ~ city_test.Total.residents, data = conc.grf,
     log = "x",
     ylab = list(expression('Log'['10']*'(projected airshed concentration, ug/m3)'), cex = 1.),
     xlab = list("Population", cex = 1.),
     pch = 8,
     col = "darkgrey"
     )
#Find the linear model; put a summary in lin.conc
#eqn s3 with h.u~1e3 and 1e-3 km^{-1} -> m{-1}
lin.conc<-summary(lm(conc.form ~ log.pop)) #summary of the linear model
alfa.conc<-round(lin.conc$coefficients[2,1:2], digits = 3) #slope and error on slope, rounded to 3 sig figs
#add line of best fit and slope
abline(coef = coef(lin.conc))
#Write out the slope and error on slope; first two elements give position on graph
text(mean(city_test$Total.residents),(min(conc.form)+0.25),
     paste("h.u =",round(hu, digits = 2)))
text(mean(city_test$Total.residents),(min(conc.form)+0.15),
     paste("Slope =",round(alfa.conc[1], digits = 2),"+/-",round(alfa.conc[2], digits = 2)))
#write out the r-square and sample size, n
text(mean(city_test$Total.residents),min(conc.form),
     paste('r-squared = ',round(lin.conc$adj.r.squared, digits = 2),"; n = ",dim(city_test)[1])
     )


#Input urban background concs from Air Quality UK
#Need to be really careful setting no data string
obs<-read.csv("AirQualityDataStatistics_tidy.csv",na.strings = "No data")
obs.pops<-read.csv("obs_urb_bgrnd_clean.csv",na.strings = " NA") 
obs.only<-data.frame(obs[,seq(2,122,2)])
obs.stats<-summary.data.frame(obs.only) #summary stats
obs.medians<-as.numeric(numextract(obs.stats[4,]))
obs.means<-colMeans(obs.only, na.rm = TRUE, dims = 1) # NB capital M!
obs.names<-colnames(obs.only)
obs.frame<-data.frame(obs.names,obs.pops$Population,log10(obs.medians))

#Plot observed points
points(log10.obs.medians. ~ obs.pops.Population, data = obs.frame, 
       type = "p",
       pch = 17, cex = 1.5)
#plot a best-fit line
obs.conc<-summary(lm(log10.obs.medians. ~ obs.pops.Population, data=obs.frame)) 
#slope and error on slope, rounded to 3 sig figs
alfa.obs<-round(obs.conc$coefficients[2,1:2], digits = 3) 
#add line of best fit and slope
abline(coef = coef(obs.conc))
text(mean(city_test$Total.residents),(max(conc.form)-0.15),
     paste("obs slope =",round(alfa.obs[1], digits = 2),"+/-",round(alfa.obs[2], digits = 2)))

#Name the points
#text(obs.pops, obs.frame$log10.obs.means., obs.names, pos = 1)

#Look at the residuals for NOx 
expected.NOx<-lin.sum$coefficients[1] + lin.sum$coefficients[2]*log.pop #Find the expected values
residjool.NOx<-expected.NOx - log.param #Calc residuals such that 'better than expected' is +ve
indx<-seq(1,length(residjool.NOx),1) #number them up
residjool.df<-data.frame(indx,residjool.NOx) #make a dataframe
plot(residjool.NOx ~ indx, data = residjool.df) #plot
abline(h=0) #add a horizontal at zero

#Look at the residuals for area
expected.area<-urb.sum.area$coefficients[1] + urb.sum.area$coefficients[2]*log.pop #Find the expected values
residjool.area<-expected.area - urb.form #Calc residuals such that 'better than expected' is +ve
indx<-seq(1,length(residjool.area),1) #number them up
residjool.area.df<-data.frame(indx,residjool.area) #make a dataframe
residjool.area.stats<-summary(residjool.area.df) #summary provides useful statistics
residjool.contribution<-residjool.area/residjool.NOx
residjool.contribution.sumry<-summary(residjool.contribution)

#plot(residjool.area ~ indx, data = residjool.area.df) #plot
#abline(h=0) #add a horizontal at zero

#Try a barplot showing both residjools
residjools.all<-data.frame(residjool.NOx,residjool.area)
residjools.all.mat<-as.matrix(residjools.all, rownames.force = NA)
residjools.all.t<-t(residjools.all.mat)
resids.sum<-residjools.all.t[1,]+residjools.all.t[2,]


#Sort the sum of the residuals and keep track of the index of each sorted point
sorted.resids.NOx<-sort(residjool.NOx, decreasing = TRUE, na.last = NA, index.return = TRUE) 
#area is matched to the sorted indices of NOx
sorted.resids.area<-residjool.area[c(sorted.resids.NOx$ix)]
#Retrieve settlement names
sorted.settlement.names<-city_test$FIRST_NAME[c(sorted.resids.NOx$ix)]

sorted.resids.sum<-sort(resids.sum, decreasing = TRUE, na.last = NA, index.return = TRUE) 
sorted.settlesum.names<-city_test$FIRST_NAME[c(sorted.resids.sum$ix)]
sorted.sum.names<-data.frame(sorted.resids.sum$ix,sorted.settlesum.names, sorted.resids.sum$x)

#chooze<-sorted.resids.NOx$x #choose one of sorted vectors
#chooze<-sorted.resids.area # ...or choose the re-jigged area vector
#barplot(height = chooze) # a sorted barplot of both residuals summed

#plot a stacked barplot
sorted.resids.names<-data.frame(sorted.settlement.names, sorted.resids.NOx$x, sorted.resids.area)
sorted.resids.all<-data.frame(sorted.resids.NOx$x, sorted.resids.area) # just the numbers
sorted.resids.mat<-as.matrix(sorted.resids.all, rownames.force = NA)
sorted.resids.t<-t(sorted.resids.mat) #transposed matrix

#Create an empty barplot - provides graphics values to pass to 'points' function
mp<-barplot(sorted.resids.t,
        xlab = list(expression('Rank by NO'['x']*'emission residual'), cex = 1.),
        ylab = list("Residual (positive = 'better than expected')", cex = 1.),
        ylim = c(-0.4, 0.3),
        col = c("white", "white"),
        border = NA,
       )
abline(h=0) #add a horizontal at zero

#Plot the NOx residjools as stairs to show +/- effect of area
points(mp, sorted.resids.NOx$x, 
       type = "s",
       lwd = 2,
       col = "red")
#Plot NOx+area residuals as a stair plot
points(mp, sorted.resids.NOx$x + sorted.resids.area, 
       type = "s",
       lwd = 2,
       col = "blue")
#add a legend
leg.txt<-c("NOx only","NOx+area")
legend(list(x = 0,y = 0),
       title = "Residuals",
       legend=leg.txt, 
       col = c("red","blue"), 
       pch = 1:2, pt.cex = 0,
       lty = 1, lwd = 2,
       merge = FALSE)
#Add some names of places
#Draw lines from text to graph
textnames = c("London","Edinburgh","Bristol","Glasgow","Birmingham","Newport")
xnames = c(21,48,75,76,112,142)
yarrows = c(0.23,0.23,0.23,-0.33,-0.33,-0.07)
ynames = c(0.25,0.25,0.25,-0.35,-0.35,-0.05)
arrows(mp[xnames], yarrows, 
       mp[xnames], 
       sorted.resids.t[1,xnames],
       code = 1, angle = 45, length = 0.05)
text(mp[xnames],ynames,textnames)


#----------------------------------------------------------------

#plot as stacked plots using par or layout functions
frame() # completes current plot
par(mfrow=c(2,1), mai=c(0.8, 1,0.2,0.5)) #control ugly white space between plots using mai
barplot(sorted.resids.NOx$x,    # REMEMBER sorted arrays can have two dimensions!
        xlab = list(expression('Rank by NO'['x']*'emission'), cex = 1.),
        ylab = list(expression('Absolute NO'['x']*'anomaly'), cex = 1.),
        ylim = c(-0.3, 0.3)
        )
barplot(sorted.resids.area,
        xlab = list(expression('As ranked by NO'['x']*'emission'), cex = 1.),
        ylab = list("half(log(area)) anomaly", cex = 1.),
        ylim = c(-0.3, 0.3)
        )

#Output the residuals sorted on NOx
write.table(sorted.resids.names, file = "sorted_residuals.txt",
            eol = "\r\n", na = "NA", dec = ".",
            row.names=TRUE,
            col.names = TRUE)

#Output the residuals sorted on sum of residuals
write.table(sorted.sum.names, file = "sorted_sum_residuals.txt",
            eol = "\r\n", na = "NA", dec = ".",
            row.names=TRUE,
            col.names = TRUE)

#Output the urban background obs to match up with populations
#write.table(obs.names, file = "obs_urb_bgrnd.txt",
#            eol = "\r\n", na = "NA", dec = ".",
#            row.names=TRUE,
#            col.names = TRUE)

